#!/usr/bin/python
# -*- coding: utf-8 -*-

'''
compressor
==========

Compresses data (e.g. averaging)
'''

import os
import math


def avg_stats(xCol, yCol, numFiles, filePrefix='', fileSuffix='', filePath='.', delim=',', outName=None):
    '''
    This function can take several data files and transfer them into a file that is formatted
    ready to be plotted by gnuplot (each line is "<x> <mean of y vals> <std of y vals>").
    Data files should be named using an index starting by 1 and all have the same prefix and/or suffix arround that index.

    In gnuplot, you could then say "plot outName smooth unique with yerrorlines"

    :param int xCol: x column
    :param int yCol: y column
    :param int numFiles: the number of files to average over
    :param string filePrefix: prefix in filenames
    :param string fileSuffix: suffix in filenames
    :param string filePath: path to files
    :param string delim: delimiter used between columns
    :param string outName: Name of the result file, defaults to <filePrefix><yCol><fileSuffix>.out
    '''

    assert os.path.exists(filePath), 'File path %s does not exist' % filePath
    if len(filePath) > 1 and not filePath.endswith("/"): filePath += "/"
    if numFiles is None:
        numFiles = len([f for f in os.listdir(filePath) if f.endswith(fileSuffix) and f.startswith(filePrefix)])
    assert numFiles, 'numFiles is zero or not set'
    assert xCol, 'xCol is not set'
    assert yCol, 'yCol is not set'

    # ---- First,  we'll collect all y values from all files
    d = {} # this will store a list of y values for each x
    for i in xrange(1, numFiles+1, 1):
        f = open(filePath + filePrefix + str(i) + fileSuffix, 'r')
        hasMoreRows = True
        while hasMoreRows:
            s = f.readline().strip().split(delim)
            if s == ['']:
                hasMoreRows = False
            else:
                # disregard comments and unsuitable lines
                if s[0].startswith('#') or len(s) < xCol or len(s) < yCol:
                    continue
                x = s[xCol-1].strip()
                if not x == '' and not d.has_key(x):
                    d[x] = []
                try:
                    # we assume that y values are numeric! Also,other
                    # errors might happen here when file is corrupted
                    d[x].append(float(s[int(yCol)-1]))
                except Exception, e:
                    print "ERROR"
        f.close()

    # ---- Then, we compute mean and standard deviation for the v values for
    #      each x and write them to target file
    if outName is None: outName = '%s%s%s%s.out' % (filePath, filePrefix, str(yCol), fileSuffix)
    out = open(outName, 'w')
    keys = d.keys()
    keys.sort()
    for x in keys:
        # -- mean
        sum = 0.0
        for y in d[x]:
            sum += y
        mean = sum / float(len(d[x]))
        # -- standard deviation (std) or standard error (ste)
        #    On the difference between them, see the very readable intro at
        #    http://ww1.cpa-apc.org:8080/publications/archives/PDF/1996/Oct/strein2.pdf
        std = 0.0
        for y in d[x]:
            std += math.pow(y - mean, 2)
        l = len(d[x])
        if l > 1:
            l -= 1
        std /= l
        std = math.sqrt(std)
        ste = std / math.sqrt(len(d[x])) # we're not using this, #40 should give a configurable choice
        out.write('%s %f %f\n' % (str(x), mean, std))

    out.close()




#!/usr/bin/python

'''
harvester
==========

This module helps to collect certain files and values from the whole data set,
based on search criteria.
Main functions to use are collect_files and collect_values.
'''

import os
from subprocess import Popen


# ---- main functions ----

def collect_files(searches, filepath, target_dir):
    ''' For each search, collect all .log-files from the data directories that matches the criteria.
        Create a subdir for each search in target_dir where the .log-files go, numbered from 1 to n.

        :param dict searches: a dict, where the keys are names of searches and values
                  are lists of key-value tuples which dirnames should match
        :param string filepath: path to data directories
        :param string target_dir: directory where the subdirs should go
        :returns: a list with searches that didn't match any folders
   '''
    failed = []

    for s in searches.keys():
        if not os.path.exists('%s/%s' % (target_dir, s)):
            os.makedirs('%s/%s' % (target_dir, s))

        foldernames = [fn for fn in os.listdir(filepath) if matches(fn, searches[s])]
        if len(foldernames) == 0:
            failed.append(s)

        copied_files = 0
        for fon in foldernames:
            filenames = [fin for fin in os.listdir('%s/%s' % (filepath, fon)) if fin.endswith('.dat')]
            for fin in filenames:
                copied_files += 1
                Popen("cp %s/%s/%s '%s/%s/log%d.dat'" % (filepath, fon, fin, target_dir, s, copied_files), shell=True).wait()
    return failed


def collect_values(filepath, delim, outfile_name, cols=[], selector='all'):
    '''
    Collect specific x/y values from a bunch of .log files (to be found via the filepath)
    and write them into a new file.

    TODO: how to pass custom selectors?

    :param string filepath: path to the log files
    :param string delim: delimiter in the data files
    :param string outfile_name: path and name of file to write into
    :param list cols: columns to select, at least one,
                      we consider the first as x, the second as y
    :param selector: method to specify how to choose lines from the data files
                     one of ['all', 'last', 'max_x', 'max_y', 'min_x', 'min_y']
    '''
    assert(selector in ['all', 'last', 'max_x', 'max_y', 'min_x', 'min_y'])
    import harvester
    selector = harvester.__getattribute__('select_%s' % selector)
    vals = []
    files = [f for f in os.listdir(filepath) if f.endswith('.dat')]
    for f in files:
        tmp = open('%s/%s' % (filepath, f), 'r')
        vals.extend(selector(tmp, cols, delim))
    out = open(outfile_name, 'w')
    for v in vals:
        out.write('%s' % str(v))
    out.close()



# ---- selectors ----

def select_all(filep, cols, delim):
    ''' select column values of all lines

    :param file filep: pointer to data file
    :param list cols: list of column indices from which to collect
    :param string delim: delimiter in the data files
    :returns: string with lines
    '''
    s = ''
    for line in [l for l in filep.readlines() if not l.startswith('#')]:
        s += '%s\n' % ' '.join([line.split(delim)[cols[i]-1].strip() for i in range(len(cols))])
    return s


def select_last(filep, cols, delim):
    ''' :returns: a string with the last values from this file and columns

    :param file filep: pointer to data file
    :param list cols: list of column indices from which to collect
    :param string delim: delimiter in the data files
    :returns: string with line
    '''
    lines = [l for l in filep.readlines() if not l.startswith('#')]
    last_line = lines[len(lines)-1].split(delim)
    return '%s\n' % ' '.join([last_line[cols[i]-1].strip() for i in range(len(cols))])


def select_max_x(filep, cols, delim):
    ''':returns: a string with the values from the line with maximal x-value

    :param file filep: pointer to data file
    :param list cols: list of column indices from which to collect
    :param string delim: delimiter in the data files
    '''
    return extreme(filep, cols, delim, sel=max, by=0)


def select_max_y(filep, cols, delim):
    ''':returns: a string with the values from the line with maximal y-value

    :param file filep: pointer to data file
    :param list cols: list of column indices from which to collect
    :param string delim: delimiter in the data files
    '''
    return extreme(filep, cols, delim, sel=max, by=1)


def select_min_x(filep, cols, delim):
    ''':returns: a string with the values from the line with minimal x-value

    :param file filep: pointer to data file
    :param list cols: list of column indices from which to collect
    :param string delim: delimiter in the data files
    '''
    return extreme(filep, cols, delim, sel=min, by=0)


def select_min_y(filep, cols, delim):
    ''':returns: a string with the values from the line with minimal y-value

    :param file filep: pointer to data file
    :param list cols: list of column indices from which to collect
    :param string delim: delimiter in the data files
    '''
    return extreme(filep, cols, delim, sel=min, by=1)


# ---- helpers ----

def matches(string, search):
    ''':returns: True if string contains all key/value pairs in search s, False otherwise
       :param string string: string to search in
       :param dict search: key-values in this dict are the search
    '''
    for k, v in search:
        if string.rfind('%s%s' % (k, v)) < 0:
            return False
    return True


def extreme(filep, cols, delim, sel=max, by=0):
    ''' Helper for selectors. Gets lines with maximal or minimal value,
        looking for those values in a column of choice.

    :param file filep: pointer to data file
    :param list cols: list of column indices from which to collect
    :param string delim: delimiter in the data files
    :param function sel: function to selct value from a list, max or min
    :param int by: the column of choice (0 for x, 1 for y), default 0

    :returns: a string with the values from the line with minimal y-value
    '''
    assert(sel in [max, min])
    assert(by in [0,1])
    lines = [l for l in filep.readlines() if not l.startswith('#')]
    vals = [float(l.split(delim)[cols[by]-1]) for l in lines]
    line = lines[vals.index(sel(vals))].split(delim)
    return '%s\n' % ' '.join([line[cols[i]-1].strip() for i in range(len(cols))])





#!/usr/bin/python

'''
tester
=======
'''

import sys
import os.path as osp
from shutil import rmtree
from subprocess import Popen

from stosim.sim import utils
from stosim.analysis import harvester


def ttest(simfolder, c, i, delim):
    '''
    Run a T-test with Gnu R

    :param string simfolder: path to simulation folder
    :param ConfigParser c: the config file where this test is described
    :param int i: number of test in that config file
    '''
    tmp_dir = '%s/tmp_tester' % simfolder

    # -- prepare data --
    # for each of the two datasets, retrieve files and then values
    sets = []
    for dset in [1,2]:
        d = utils.decode_search_from_confstr(c.get('ttest%i' % i, 'set%i' % dset), sim = c.get('meta', 'name'))
        if d.has_key('_name') and d.has_key('_col') and d['_col'].isdigit():
            sets.append(d['_name'])
            searches = {d['_name'] : [(k, d[k]) for k in d.keys() if not k in ['_name', '_col']]}
            harvester.collect_files(searches, "%s/data" % simfolder, tmp_dir)
            #TODO: custom selectors?
            if not d.has_key('_select'):
                d['_select'] = 'all'
            harvester.collect_values("%s/%s" % (tmp_dir, d['_name']),
                                     delim,
                                     '%s/%s.dat' % (tmp_dir, d['_name']),
                                     cols=[int(d['_col'])],
                                     selector=d['_select']
                                    )
        else:
            print '[StoSim] Warning: Incomplete T-test specification for test %i in Experiment %s, dataset number %i. '\
                  'Specify at least _name and _col.' % (i, c.get('meta', 'name'), dset)

    # -- run test --
    if osp.exists('%s/%s.dat' % (tmp_dir, sets[0])) and osp.exists('%s/%s.dat' % (tmp_dir, sets[1])):
        if c.has_option('ttest%i' %i, 'custom-script'):
            custom_script = c.get('ttest%i' % i, 'custom-script')
            if not osp.exists(custom_script):
                print "[StoSim] Cannot find custom script at [%s]. Aborting ..." % (custom_script)
                print
                return
            print '[StoSim] Using custom script at %s' % custom_script
            Popen('cp %s %s/ttest.r' % (custom_script, tmp_dir), shell=True).wait()
        else:
            rscript = open('%s/ttest.r' % tmp_dir, 'w')
            rscript.write("%s <- read.table('%s.dat')\n" % (sets[0], sets[0]))
            rscript.write("%s <- read.table('%s.dat')\n" % (sets[1], sets[1]))
            rscript.write("t.test(%s,%s)" % (sets[0], sets[1]))
            rscript.close()
        Popen('cd %s; R --vanilla --silent < ttest.r' % tmp_dir, shell=True).wait()

    if osp.exists(tmp_dir) and not '-k' in sys.argv:
        rmtree(tmp_dir)






'''
utils
=====
'''

#TODO: refactor this module:
# 1. make this a submodule, maybe residing in the main dir, then three files:
#    sim.py, remote.py and analysis.py (the latters would have at least decode_search_from_confstr)
# 2. There should be classes (as own files?) to represent the two major confs: stosim.conf and 
#    remote.conf. This would make code cleaner. Instantiate them with the
#    location of the conf and then put all access and check functions in them
#    Notes:
#    - They should make the original ConfigParser available as conf.cp 
#    - get_main_conf could maybe go to __init__
# 3. Some functions in stosim.py could go in one of those subfiles, thus making
#    that mode readable as well (not sure yet where the primary functionality
#    should go, maybe sim/__init__.py, analysis/__init__.py and so on, or
#    classes? 
# 4. While doing this, look out for PEP8 compatibility

import sys
import os
import os.path as osp
import re
from ConfigParser import ConfigParser
from ConfigParser import NoOptionError, NoSectionError, ParsingError
try:
	import argparse
except ImportError:
	print("[StoSim] Import error: You need Python 2.7+ (you can, however, copy the argparse module inside your local directory.")
	sys.exit(1)



def read_args():
    """
    read in cmd line arguments for StoSim, print usage if something is unexpected
    :returns: arg object returned by argparse
    """
    parser = argparse.ArgumentParser(description='StoSim is an open-source toolkit for running parameterised stochastic simulations and analysing them.\
                                                  Please visit http://homepages.cwi.nl/~nicolas/stosim')
    parser.add_argument('--folder', metavar='PATH', default='.', help='Path to simulation folder (this is where you keep your stosim.conf), defaults to "."')
    parser.add_argument('--simulations', metavar='NAME', nargs='*', help='names of subsimulations (the filenames of their configuration files without the ".conf" ending).')
    parser.add_argument('--run', action='store_true', help='Only run, do not analyse.')
    parser.add_argument('--list', action='store_true', help='List number of runs made so far, per configuration.')
    parser.add_argument('--more', action='store_true', help='Add more runs to current state of config and data.')
    parser.add_argument('--plots', metavar='FIGURE', nargs='*', type=int, help='Make plots (needs gnuplot and eps2pdf installed). Add indices of figures as arguments if you only want to generate specific ones.')
    parser.add_argument('--ttests', action='store_true', help='Run T-tests (needs Gnu R installed).')
    parser.add_argument('-k', action='store_true', help='keep tmp analysis files.')
    parser.add_argument('-d', action='store_true', help='delete old data without confirmation.')

    return parser.parse_args()


def check_conf(simfolder):
    """
    check if stosim.conf contains all necessary sections and options
    :param string simfolder: relative path to simfolder
    """
    conf = ConfigParser()
    try:
        conf.read("%s/stosim.conf" % simfolder)
    except ParsingError, e:
        print "[StoSim] %s" % e
        sys.exit(2)

    if not osp.exists("%s/stosim.conf" % simfolder):
        print "[StoSim] Cannot find stosim.conf in the folder '%s' - Exiting..." % simfolder
        sys.exit(2)

    if not conf.has_section('meta') or not conf.has_option('meta', 'name'):
        print "[StoSim] You need to tell me a name for this simulation. \
            Please define an option called 'name' in a section called 'meta'."
        sys.exit(2)

    if not conf.has_section('control') or not conf.has_option('control', 'executable'):
        print "[StoSim] You need to tell me what script to execute. \
            Please define an option called 'executable' in a section called 'control'."
        sys.exit(2)

    if not conf.has_section('params'):
        print "[StoSim] Warning: You have not defined a 'params' - section."


def get_main_conf(simfolder):
    """ Return ConfigParser object read from main conf, with all relevant
        subsimulation configs set

        :param string simfolder: relative path to simfolder
        :returns: ConfigParser object
    """
    conf = ConfigParser()
    try:
        assert(osp.exists('%s/stosim.conf' % simfolder))
    except AssertionError:
        print "[StoSim] Cannot find stosim.conf in the folder '%s' - Exiting ..." % simfolder
        sys.exit(2)
    conf.read("%s/stosim.conf" % simfolder)

    if not conf.has_section('meta'):
        conf.add_section('meta')
    try:
        def_user = os.getlogin()
    except OSError, e:
        def_user = os.getenv('USER')
    for (sec, opt, default) in\
            [('meta', 'name', 'A simulation run by StoSim'),\
             ('meta', 'maintainer', def_user),\
             ('control', 'scheduler', 'fjd'),\
             ('control', 'runs', '1')]:
        if not conf.has_option(sec, opt):
            conf.set(sec, opt, default)

    args = read_args()
    if args.simulations:
        if not conf.has_section('simulations'):
            print "[StoSim] You cannot use the '--simulations' cmd line"\
                  " parameter if you do not have the [simulations] section"\
                  " in stosim.conf"
            sys.exit(2)
        conf.set('simulations', 'configs', ','.join(args.simulations))
    if conf.has_section('simulations'):
        for c in [cf.strip() for cf in conf.get('simulations', 'configs').split(',')]:
            if not osp.exists("%s/%s.conf" % (simfolder, c)):
                print "[StoSim] Warning: The file %s.conf does not exist!" % c

    return conf

def get_combined_conf(simfolder):
    ''' 
        Make a configuration with all used params and their values - this 
        means we gathe all param values from eventual subconfs,
        next to stosim.conf
        
        :param string simfolder: relative path to simfolder
        :returns: ConfigParser combined conf

    '''
    conf = get_main_conf(simfolder)
    if conf.has_section('simulations'):
        configs = conf.get('simulations', 'configs').split(',')
        for c in [cf.strip() for cf in configs]:
            subconf = ConfigParser()
            subconf.read('{}/{}.conf'.format(simfolder, c))
            for p in subconf.options('params'):
                if conf.has_option('params', p):
                    both = conf.get('params', p).split(',')
                    both.extend(subconf.get('params', p).split(','))
                    conf.set('params', p, ','.join(set(both)))
                else:
                    conf.set('params', p, subconf.get('params', p))
    return conf


def get_scheduler(simfolder):
    """ get the scheduler (fjd or pbs)

        :param string simfolder: relative path to simfolder
        :returns: string scheduler
    """
    stosim_conf = get_main_conf(simfolder)
    if not stosim_conf.has_option('control', 'scheduler'):
        scheduler = "fjd"
    else:
        scheduler = stosim_conf.get('control', 'scheduler')
    if not scheduler in ('fjd', 'pbs'):
        print("[StoSim] {} is not a valid scheduler.".format(scheduler))
        scheduler = "fjd"
    return scheduler


def get_interval(simfolder):
    """ get the interval for FJD  workers and disoatchers to check the queue,
        in seconds (fractions of seconds are allowed).

        :param string simfolder: relative path to simfolder
        :returns: float interval
    """
    stosim_conf = get_main_conf(simfolder)
    if not stosim_conf.has_option('control', 'fjd-interval'):
        interval = .2
    else:
        interval = stosim_conf.getfloat('control', 'fjd-interval')
    return interval


def get_jobtime(simfolder):
    """ get the expected maximal jobtime for PBS jobs,
        the format is HH:MM:SS

        :param string simfolder: relative path to simfolder
        :returns: string jobtime
    """
    stosim_conf = get_main_conf(simfolder)
    if not stosim_conf.has_option('control', 'pbs-jobtime'):
        jobtime = '00:05:00'
    else:
        jobtime = stosim_conf.getfloat('control', 'pbs-jobtime')
    return jobtime


def make_simdir_name(simfolder):
    '''
    Make the name for a simulation dir from the simulation name and the name of conf files
    [This and make_screen_name need overhaul and a common approach when
    a session management is implemented]

    :param string simfolder: relative path to simfolder
    '''
    stosim_conf = get_main_conf(simfolder)
    sim_name = stosim_conf.get('meta', 'name')
    simdir_name = 'main'
    subsims = get_subsimulation_names(stosim_conf)
    if subsims != ['']:
        simdir_name = '_'.join(subsims)
    rx = re.compile('\W+')
    return '%s_%s' % (rx.sub('_', sim_name).strip(), simdir_name)


def get_subsimulation_names(conf):
    ''' get sub-simulation config names.

        :param ConfigParser conf: main configuration
        :returns: A list with names. If no subsimulations are configured, the list will have an empty string as only element
    '''
    sim_names = ['']
    if 'simulations' in conf.sections() \
            and conf.has_option('simulations', 'configs')\
            and conf.get('simulations', 'configs') != '':
        sim_names = conf.get('simulations', 'configs').split(',')
    sim_names = [s.strip() for s in sim_names]
    return sim_names


def get_simulation_name(conf_filename, fallback):
    ''' The user can give a pretty name to the simulation under [meta], this function returns it.

        :param string conf_filename: name of the config file for the simulation
        :param string fallback: return this if the user didn't specify any
        :returns: string pretty name
    '''
    conf = ConfigParser()
    conf.read(conf_filename)
    if conf.has_option('meta', 'name'):
        return conf.get('meta', 'name').replace(' ', '_')
    else:
        return fallback


def ensure_name(simfolder):
    ''' make sure we have the actual name of the folder and not just '.'

        :param string simfolder: relative path to simfolder
        :returns: the full name (without the path to it)
    '''
    if simfolder == '.':
        simfolder = osp.abspath(osp.curdir).split('/')[-1:][0]
    return simfolder.strip('/')


def runs_in_folder(simfolder, fname):
    ''' :returns: number of runs that have been made in this data folder
        :param string simfolder: relative path to simfolder
    '''
    fpath = "%s/data/%s" % (simfolder, fname)
    if not os.path.exists(fpath):
        return 0
    logfiles = [f for f in os.listdir(fpath) if f.startswith('log') and f.endswith('.dat')]
    if len(logfiles) == 0:
        return 0
    log_numbers = [int(f.split('.')[0][3:]) for f in logfiles]
    return max(log_numbers)


def get_relevant_confs(simfolder):
    ''' :returns: ConfigParser objects for all config files being used
        :param string simfolder: relative path to simfolder
    '''
    conf = get_main_conf(simfolder)
    relevant_confs = [conf]
    if conf.has_section('simulations'):
        for subsim in conf.get('simulations', 'configs').split(','):
            c = ConfigParser()
            filename = subsim.strip()
            c.read("%s/%s.conf" % (simfolder, filename))
            c.set('meta', '_subconf-filename_', filename)
            relevant_confs.append(c)
    return relevant_confs


def get_delimiter(conf):
    ''' get the delimiter used in the log files between values

        :param ConfigParser conf: the main config file
        :returns: the user-defined delimiter or the assumed default one (,)
    '''
    if conf.has_option('control', 'delimiter'):
        d = conf.get('control', 'delimiter')
        d = d.replace('\\t', '\t')
        return d
    else:
        return ','


def decode_search_from_confstr(s, sim=""):
    ''' Make dict out of configuration string that describes a search for a sub-dataset (for plots, ttests)

        :param string s: a string of comma-separated key-value pairs
        :returns: dictionary made from the string
    '''
    d = {}
    # prepare to deal with special chars
    s = s.replace('\\\\,', '\\ ,')
    s = s.replace('\\\\:', '\\ :')
    s = s.replace('\\,', "#COMMA#")
    s = s.replace('\\:', "#COLON#")
    s = s.replace('\\\\', '\\')

    for item in s.split(','):
        if item == '':
            continue
        try:
            k, v = item.split(':')
            v = v.replace("#COMMA#", ',')
            v = v.replace("#COLON#", ':')
        except:
            print '[StoSim] Misconfiguration in Experiment %s while parsing "%s". This is the plot configuration, please check: "%s" ... ' % (sim, item, s)
            continue
        d[k.strip()] = v.strip()
    return d



#!/usr/bin/python

'''
plotter
=======
'''

import sys
import os
import os.path as osp
from subprocess import Popen
from shutil import rmtree

from stosim.analysis import compressor
from stosim.analysis import harvester


with_names = False # if to have title in PDF
show_pdfs = False  # if to show PDF right after creation
pdf_viewer = 'okular'
tmp_dir = 'tmp_plotter'


def plot(filepath='', delim=',', outfile_name='', name='My simulation',\
         xcol=1, x_label='iteration', y_label='value', x_range=None,  y_range='[0:10]',\
         use_y_errorbars=False, errorbar_every=1, infobox_pos='top left', use_colors=True,\
         use_tex=False, line_width=6, font_size=22, custom_script='', plots=[]):
    '''
    Make plots for specific portions of data in one figure.

    For each graph, this function ...

    - selects folders with data files depending on parameter values you provide
    - collects all log files contained in them in a temporary folder
    - averages over the contents or just selects values from them
    - makes a (gnu)plot out of that, with yerrorbars if you want

    The output is one PDF file.
    Creating PDF for this yields far nicer linetypes.
    Also, this makes different linetypes (e.g. dashes/dots) possible,
    bcs papers are often printed b/w.

    In addition to gnuplot, you need epstopdf installed.

    :param string filepath: path to data
    :param string delim: delimiter used between columns
    :param string outfile_name: name of the PDF file you want to make
    :param string name: Title of the simulation
    :param int xcol: column >= 1
    :param string x_label: label on x axis
    :param string y_label: label on y axis
    :param string x_range: the range of values for x axis, in the form of
           "[a:b]", defaults to None, meaning that gnuplot should take to the actual
           value range of x values
    :param string y_range: the range of values for y axis, defaults to '[0:10]'
    :param boolean use_y_errorbars: True if errorbars should be shown, default is False
    :param int errorbar_every: show an errorbar every x steps, defaults to 1
    :param string infobox_pos: where the infobox should go, e.g. 'bottom left',
           defaults to 'top left' when given empty
    :param boolean use_colors: whether to use colors or be b+w, defaults True
    :param boolean use_tex: whether to use enhanced mode, which interpretes
           tex-like encoding (e.g. subscript, math, greek symbols), defaults to False
    :param int line_width: line width in pixels, defaults to 6
    :param int font_size: font size (a number), defaults to 22
    :param list plots: list of plot descriptions, defaults to empty list
    '''

    print '[StoSim] Preparing %s: ' % outfile_name ,

    # make sure tmp dir exists
    if not os.path.exists(tmp_dir):
        os.mkdir(tmp_dir)

    # ---- collect relevant data for each requested plot ----
    # get relevant files
    searches = {}
    for p in plots:
        # make sure no old data is around
        plot_dir = '%s/%s' % (tmp_dir, p['_name'])
        if os.path.exists(plot_dir):
            if len(os.listdir(plot_dir)) > 0:
                Popen('rm -r %s/*' % plot_dir, shell=True).wait()
        searches[p['_name']] = [(k, p[k]) for k in p.keys() if not k.startswith('_')]
        print '%s ' % p['_name'],
    failed = harvester.collect_files(searches, filepath, tmp_dir)
    print
    # handle errors
    init_plots_num = len(plots)
    if len(failed) > 0:
        print "[StoSim] WARNING: Selectors %s didn't match any folders!" % ','.join(failed)
        for fail in failed:
            for p in plots:
                if p['_name'] == fail:
                    plots.remove(p)
    if len(failed) == init_plots_num:
        print "[StoSim] In fact, no selectors of this figure matched anything. Aborting ..."
        print
        return

    # ---- prepare data  ----
    for p in plots:
        if p['_type'] == 'line':
            compressor.avg_stats(xcol, int(p['_ycol']), None, filePrefix='log',
                      fileSuffix='.dat', filePath='%s/%s' % (tmp_dir, p['_name']),
                      delim=delim, outName='%s/%s/all.dat' % (tmp_dir, p['_name']))
        else: # scatter
            if not p.has_key('_select'):
                p['_select'] = 'all'
            harvester.collect_values('%s/%s' % (tmp_dir, p['_name']), delim, '%s/%s/all.dat' \
                           % (tmp_dir, p['_name']), cols=[xcol, int(p['_ycol'])],
                           selector=p['_select'] )

    # ---- plot the results ----
    if custom_script != "":
        if not osp.exists(custom_script):
            print "[StoSim] Cannot find custom script at [%s]. Aborting ..." % (custom_script)
            print
            return
        print '[StoSim] Using custom script at %s' % custom_script
        Popen('cp %s %s/plot.gnu' % (custom_script, tmp_dir), shell=True).wait()
    else:
        e = ''
        if use_tex:
            e = 'enhanced'

        c = 'monochrome'
        if use_colors:
            c = 'color'

        # write gnuplot code
        gnu = "set terminal postscript %s eps %s dashed lw %d rounded %d;\n" % (e, c, int(line_width), int(font_size))
        gnu += "set output '%s.eps';\n" % name
        if x_range:
            gnu += "set xrange %s;\n" % x_range
        gnu += "set yrange %s;\n" % y_range
        if with_names:
            gnu += "set title '%s';" % name
        gnu += "set xlabel '%s';\n" % x_label
        gnu += "set ylabel '%s';\n" % y_label
        gnu += "set key %s spacing 1;\n" % infobox_pos
        gnu += "plot "
        # first the actual plots
        num = 1
        for p in plots:
            smu = ''
            if p['_type'] == 'line':
                smu = 'smooth unique'
            gnu += "'%s/all.dat' %s " % (p['_name'], smu)
            gnu += " title '%s' lt %d" % (p['_name'], num)
            if num < len(plots):
                gnu += ','
            num += 1
        # then the errorbars (this way, they don't reserve the linestyles)
        if use_y_errorbars:
            offset = 1
            offset_step = max(1, int(errorbar_every) / 5)
            offtxt = ''
            if int(errorbar_every) > 1:
                offtxt = 'every %d::%d' % (int(errorbar_every), offset)
            num = 1
            gnu += ","
            for p in plots:
                gnu += "'%s/all.dat' %s with yerrorbars title '' lt %d" \
                        % (p['_name'], offtxt, num)
                if num < len(plots):
                    gnu += ','
                num += 1
                #TODO: this isn't nice for everyone... can maybe be derived from xrange somehow ... ?
                offset += offset_step

        # execute gunplot code
        gnuf = open('%s/plot.gnu' % tmp_dir, 'w')
        gnuf.write(gnu)
        gnuf.close()

    # generate PDF and maybe show it
    print '[StoSim] Plotting %s' % outfile_name
    Popen('cd %s; gnuplot plot.gnu; epstopdf %s.eps; cd ..' % (tmp_dir, name), shell=True).wait()
    Popen('cp %s/%s.pdf %s' % (tmp_dir, name, outfile_name), shell=True).wait()
    print

    if osp.exists(tmp_dir) and not '-k' in sys.argv:
        rmtree(tmp_dir)

    if show_pdfs:
        Popen('%s %s' % (pdf_viewer, outfile_name), shell=True).wait()




#!/usr/bin/python

'''
job_creator
=====

This module sets up the stage - with it, you can make a configuration file
for each mix of parameter settings (for each job) that needs to be run.
'''

import sys
import os
from ConfigParser import ConfigParser
from shutil import rmtree

import utils



def create(main_conf, simfolder, limit_to={}, scheduler='fjd', more=False):
    """
    Writes a conf file for each run that the parameters in conf suggest.

    :param ConfigParser conf: main configuration
    :param string simfolder: relative path to simfolder
    :param dict limit_to: dict of configuration settings we should limit to (default is an empty dict)
    :param boolean more: True if runs should be appended to existing data (default is False)
    """

    # these hold all parameter names and the different value configurations, per simulation
    #comb_options = [] # a list of param names
    #comb_values = []  # a list of lists with param values - one per job

    # ---------------------------------------------------------------------------------------------
    # define some helpful functions

    def combiner(lists):
        '''
        :param list lists: list of lists with inputs
        :returns: all possible combinations of the input items in a nested list
        '''
        res = [[]]
        for l in lists:
            tmp = []
            for i in l:
                for j in res:
                    tmp.append(j+[i])
            res = tmp
        return res


    def get_options_values(sim_params, limit_to):
        '''
        :param dict sim_params: dict of configuration for one simulation
        :param dict limit_to: dict of configuration settings we should limit to
        :returns: a list of option names and lists of values to use in every possible combination in one simulation.
        '''
        # these hold all parameter names and the different value configurations, per simulation
        comb_options = [] # a list of param names
        comb_values = []  # a list of lists with param values - one per job

        for k, v in sim_params.iteritems():
            comb_options.append(k)
            comb_values.append(v)
        comb_values = combiner(comb_values)

        # if we filter the todo-list, remove some configs
        num = len(comb_values)
        for i in xrange(num-1, -1, -1):
            tmpcv = comb_values[i]
            matches = True
            for k, vals in limit_to.iteritems():
                if not str(tmpcv[comb_options.index(k)]) in vals:
                    matches = False
            if not matches:
                comb_values.pop(i)

        return comb_options, comb_values

    def mk_option((opt, isint, sec), sim_conf):
        ''' helper function to copy values in conf file from meta and control section '''
        from_conf = sim_conf.has_option(sec, opt) and sim_conf or main_conf
        getter = isint and getattr(from_conf, 'getint') or getattr(from_conf, 'get')
        return '{}:{}\n'.format(opt, getter(sec, opt))

    def write_job(values, sim='', run=1):
        '''
        Write a job conf with these values for this sim
        '''
        job_name = "sim{}_".format(sim)
        for i in range(len(values)):
            job_name += "{}{}".format(options[i], values[i])
            if i < len(values) - 1:
                job_name += "_"
        job_conf_filename = '{}/jobs/{}_run{}.conf'.format(simfolder, job_name, run)
        job_conf = open(job_conf_filename, 'w')

        if more:
            run += utils.runs_in_folder(simfolder, job_name)

        # these meta sections settings are tricky - they might be overwritten 
        # per simulation and we might want to start where we left off
        sim_conf = ConfigParser(); sim_conf.read("%s/%s.conf" % (simfolder, sim))
        job_conf.write('[meta]\n')
        for dat in [(opt, isint, 'meta') for (opt, isint) in [('name', 0), ('maintainer', 0)]]:
            #if main_conf.has_option('meta', opt):
            job_conf.write(mk_option(dat, sim_conf))
        
        job_conf.write('\n[control]\n')
        exe = mk_option(('executable', 0, 'control'), sim_conf)
        exe = 'executable: {}/{}\n'.format(simfolder, exe.split(':')[1].strip().strip('./'))
        job_conf.write(exe) 
        logfile = '{}/data/{}/log{}.dat'.format(simfolder, job_name, run)
        if not os.path.exists('{}/data/{}'.format(simfolder, job_name)):
            os.mkdir('{}/data/{}'.format(simfolder, job_name))
        job_conf.write('logfile: {}\n'.format(logfile))
        if main_conf.has_option('seeds', str(run)):
            seed = main_conf.get('seeds', str(run)) 
            job_conf.write('seed:{}\n'.format(seed)) 
        job_conf.write('\n')

        # write param values
        job_conf.write('[params]\n')
        for i in range(len(values)):
            job_conf.write('{}:{}\n'.format(options[i], values[i]))

        job_conf.flush()
        job_conf.close()

        if scheduler == 'pbs':
            pbs_job = '''# Shell for the job:
#PBS -S /bin/bash
# request 1 node, {cores} cores
#PBS -lnodes=1:cores{cores}
# job requires at most n hours wallclock time
#PBS -lwalltime={maxtime}

{cmd} {logfile} {jobconf}'''.format(cmd=main_conf.get('control', 'executable'), cores=1, 
                          maxtime='00:30:00', logfile=logfile,
                          jobconf=job_conf_filename)
            pbs_job_file = open('{}/jobs/{}_run{}.pbs'.format(simfolder, job_name, run), 'w')
            pbs_job_file.write(pbs_job)
            pbs_job_file.close()

    # ---------------------------------------------------------------------------------------------
    # now let's get going

    # get parameters from subsimulations: combine them with our normal params
    default_params = {}
    for param in main_conf.options('params'):
        default_params[param] = [v.strip() for v in main_conf.get('params', param).split(',')]
    simulations = {'': default_params}
    if 'simulations' in main_conf.sections() and main_conf.get('simulations', 'configs') != '':
        simulations = {}
        for sim in main_conf.get('simulations', 'configs').split(','):
            sim = sim.strip()
            simulations[sim] = default_params.copy()
            sim_conf = ConfigParser()
            sim_job_name = "%s/%s.conf" % (simfolder, sim)
            if not os.path.exists(sim_job_name):
                print "[StoSim] Error: Can't find %s !" % sim_job_name
                sys.exit()
            sim_conf.read(sim_job_name)
            if sim_conf.has_section('params'):
                for param in sim_conf.options('params'):
                    simulations[sim][param] = [v.strip() for v in sim_conf.get('params', param).split(',')]

    # now write all the conf files, once for each simulation and once for each seed
    for sim in simulations:
        options, values = get_options_values(simulations[sim], limit_to)
        
        for _ in range(len(values)):
            # get a set of unique values 
            act_values = values.pop()
            runs = main_conf.getint('control', 'runs')
            for run in xrange(1, runs+1):
                write_job(act_values, sim=sim, run=run)
                
